// ******************************************************************************
// @file    boot_common.c
// @author  Milandr Application Team
// @version V1.1.0
// @date    04/09/2024
// @brief   Common functions.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"
#include "boot_common.h"
#include "boot_stage_2.h"

// CLIC setup and Trap_IRQHandler moved to startup code.
// ------------------------------------------------------------------------------
//__TRAP_HANDLER_ALIGNED __INTERRUPT_MACHINE void Trap_IRQHandler(void)
//{
//}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void bkp_set_fpor(void)
{
    uint32_t tmp;

    // Unlock BKP write protection, set fpor to BKP fpor bit.
    // Finally lock BKP write protection.

    MDR_BKP->WPR = 0x8555AAA1;
    tmp = MDR_BKP->LDO;
    tmp &= ~(BKP_LDO_SW_OFF_Msk | BKP_LDO_TAMPF_Msk);
    tmp |= BKP_LDO_FPOR;
    MDR_BKP->LDO = tmp;
    MDR_BKP->WPR = 0;
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
inline void __attribute__((always_inline)) bkp_write_mode(uint32_t mode)
{
    uint32_t tmp;

    // Set AND mask over mode, because mode field is 2-bit in BKP.

    mode &= (BKP_LDO_MODE_Msk >> BKP_LDO_MODE_Pos);

    // Unlock BKP write protection, write boot mode to BKP mode field.
    // Finally lock BKP write protection.

    MDR_BKP->WPR = 0x8555AAA1;
    tmp = MDR_BKP->LDO;
    tmp &= ~(BKP_LDO_SW_OFF_Msk | BKP_LDO_TAMPF_Msk | BKP_LDO_MODE_Msk);
    tmp |= mode << BKP_LDO_MODE_Pos;
    MDR_BKP->LDO = tmp;
    MDR_BKP->WPR = 0;
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void run_app(uint32_t entry_point)
{
    ((void (*)(void)) entry_point)();
}
// ------------------------------------------------------------------------------

