// ******************************************************************************
// @file    boot_stage_2.c
// @author  Milandr Application Team
// @version V1.1.0
// @date    04/09/2024
// @brief   Main program body of SPD bootloader for MCU's with CHIP_ID = 217.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"

#include "boot_stage_2.h"
#include "boot_stage_3_flash.h"
#include "boot_common.h"
#include "system_MDR32VF0xI_it.h"

#include <stdbool.h>

// ------------------------------------------------------------------------------
// Main function
int main(void)
{
    // Core clock after startup code on HSI (8MHz). Peripheral clock on HSI (8MHz).
    // After startup code enabled RST_CLK, BKP, PORTD.

    // CLIC_SetTrapVector(CLIC_PRIVILEGE_MODE_IRQ_M, Trap_IRQHandler); // CLIC setup and Trap_IRQHandler moved to startup code.
    setup_manufacture_trimings();
    bkp_write_mode(mode_flash);
    bkp_set_fpor();

    // Disable clocks. PER2 clocks according product specification by reset.
    rst_clk_per2_clear(RST_CLK_PER2_CLOCK_PCLK_EN_BKP | RST_CLK_PER2_CLOCK_PCLK_EN_PORTD);
    run_app(FLASH_BASE);

    while (true);   // Deadlock.
    return (0);		// unreachable code.
}
// ------------------------------------------------------------------------------
